@@ -2,6 +2,7 @@
 (* $Id: xHTML.ml,v 1.31 2005/06/20 17:57:58 ohl Exp $
 
    Copyright (C) 2004 by Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
+   Copyright (C) 2007 by Vincent Balat, Gabriel Kerneis, CNRS, Universit Paris Diderot
 
    XHTML is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by 
@@ -23,7 +24,7 @@
    BUT: This hack is not very clean because I don't think I follow
    Thorsten Ohl's conventions.
    I verified lots of types but not all and I am not sure that it
-   corresponds exactly to the dtd. (Actually, I am almost sur that it doesn't).
+   corresponds exactly to the dtd. (Actually, I am almost sure that it doesn't).
    This module would deserve a complete verification and to be cleaned 
    following Ohl's conventions.
    Vincent Balat
@@ -149,7 +150,7 @@ module type T =
     type linktypes =
         [ `Alternate | `Appendix | `Bookmark | `Chapter | `Contents
         | `Copyright | `Glossary | `Help | `Index | `Next | `Prev
-        | `Section | `Start | `Stylesheet | `Subsection] list
+        | `Section | `Start | `Stylesheet | `Subsection | `Other of string] list
 (** Authors may use the following recognized link types, listed here with
     their conventional interpretations. A LinkTypes value refers to a
     space-separated list of link types. White space characters are not
@@ -206,7 +207,10 @@ module type T =
          Refers to a bookmark. A bookmark is a link to a key entry point within
          an extended document. The title attribute may be used, for example, to
          label the bookmark. Note that several bookmarks may be defined in each
-         document.}} *)
+         document.}
+      {- [`Other]:
+         refers to any other type (for example [icon] or [shortcut]).
+         }} *)
 
     type mediadesc =
         [ `All | `Aural | `Braille | `Handheld | `Print
@@ -304,7 +308,7 @@ module type T =
 (** Arbitrary textual data, likely meant to be human-readable. *)
 
     type uri (* I abstract this for Ocsigen -- VB *)
-    val make_uri_from_string : string -> uri
+    val uri_of_string : string -> uri
 (** A Uniform Resource Identifier, as per RFC2396.
     @see <http://www.ietf.org/rfc/rfc2396.txt> RFC2396 *)
 
@@ -559,6 +563,8 @@ module type T =
     val a_button_type : [< `Button | `Submit | `Reset ] ->
       [>`Button_Type] attrib
 
+    val a_label : text -> [> `Label ] attrib
+
 
 (** {2 5.6. Table Modules} *)
     
@@ -781,6 +787,14 @@ module type T =
     val pcdata : string -> [>`PCDATA] elt
     val entity : string -> [>`PCDATA] elt
     val space : unit -> [>`PCDATA] elt
+    val cdata : string -> [`PCDATA] elt (* GK *)
+    val cdata_script : string -> [`PCDATA] elt (* GK *)
+    val cdata_style : string -> [`PCDATA] elt (* GK *)
+(**/**)
+    val unsafe_data : string -> [`PCDATA] elt
+(**/**)
+
+
     
 (** {2 Text} *)
     
@@ -879,7 +893,7 @@ module type T =
 
 (** {3 Forms} *)
     
-(** Generic forms. WARNING: I have done this very quickly! This is not valid! -- VB *) 
+(** Generic forms. WARNING: If you find a bug or if something is missing please send a bug report to the Ocsigen project! -- VB *) 
     val form : action:uri ->
       ([< common | `Enctype | `Method | `Name_01_00 | `Target | `Accept_charset | `Accept ],
        [< block_sans_form | `Fieldset ], [>`Form]) plus
@@ -887,8 +901,9 @@ module type T =
   | `Src | `Tabindex | `Input_Type | `Value | `Disabled | `Readonly | `Alt | `Accept | `Usemap ], [>`Input]) nullary
     val label : ([< common | `Accesskey | `For ],
                  [< `PCDATA | inline_sans_label ], [>`Label]) star
-    val optgroup : ([< common | `Disabled | `Label ],
-                  [< `Option ], [>`Optgroup]) plus
+    val optgroup : label:text ->
+      ([< common | `Disabled ],
+       [< `Option ], [>`Optgroup]) plus
     val option : ([< common | `Selected | `Value | `Disabled | `Label ],
                   [< `PCDATA ], [>`Option]) unary
     val select : ([< common | `Multiple | `Name | `Size | `Tabindex | `Disabled ],
@@ -1017,6 +1032,10 @@ module type T =
 (** [~encoding] is the official name of the external character set encoding that
     is used by [outs : string -> unit]. *)
 
+    val doctype :
+        [< `HTML_v03_02 | `HTML_v04_01 | `XHTML_01_00 | `XHTML_01_01 ]
+      -> string
+ 
     val output : ?encode:(string -> string) -> ?encoding:string -> 
       (string -> unit) -> html -> unit
 
@@ -1070,12 +1089,6 @@ module type T =
     val toelt : 'a elt -> XML.elt
     val toeltl : 'a elt list -> XML.elt list
 
-    val ocsigen_print : 
-        ?width:int -> ?encode:(string -> string) -> [ `Html ] elt -> string
-
-    val ocsigen_xprint : 
-        ?width:int -> ?encode:(string -> string) -> 'a elt list -> string
-
   end
 
 (** An alias for XHTML 1.1 (for symmetry):
@@ -1151,7 +1164,7 @@ module Version =
     type linktypes =
         [`Alternate | `Appendix | `Bookmark | `Chapter | `Contents
         | `Copyright | `Glossary | `Help | `Index | `Next | `Prev
-        | `Section | `Start | `Stylesheet | `Subsection] list
+        | `Section | `Start | `Stylesheet | `Subsection | `Other of string] list
     type mediadesc =
         [ `All | `Aural | `Braille | `Handheld | `Print
         | `Projection | `Screen | `TTY | `TV ] list
@@ -1164,7 +1177,7 @@ module Version =
     type text = string
     type uri = string
     type uris = uri (* space-separated *)
-    let make_uri_from_string s = s
+    let uri_of_string s = s
 
     let color_attrib name value =
       string_attrib name
@@ -1223,6 +1236,7 @@ module Version =
       | `Start -> "start"
       | `Stylesheet -> "stylesheet"
       | `Subsection -> "subsection"
+      | `Other t -> t
 
     let linktypes_attrib name linktypes =
       string_attrib name
@@ -1300,6 +1314,8 @@ module Version =
     let a_maxlength = int_attrib "maxlength"
     let a_name = string_attrib "name"
 
+
+
 (* XHTML 1.0 allows the name attribute for more elements:*)
     let a_name_01_00 = string_attrib "name"
 
@@ -1449,6 +1465,39 @@ module Version =
 
     let space () = entity "nbsp"
     
+    let cdata s = (* GK *)
+      (* For security reasons, we do not allow "]]>" inside CDATA
+         (as this string is to be considered as the end of the cdata)
+       *)
+      let s' = "\n<![CDATA[\n"^
+        (Netstring_pcre.global_replace 
+           (Netstring_pcre.regexp_string "]]>") "" s)
+        ^"\n]]>\n" in
+      XML.EncodedPCDATA s'
+    
+    let cdata_script s = (* GK *)
+      (* For security reasons, we do not allow "]]>" inside CDATA
+         (as this string is to be considered as the end of the cdata)
+       *)
+      let s' = "\n//<![CDATA[\n"^
+        (Netstring_pcre.global_replace 
+           (Netstring_pcre.regexp_string "]]>") "" s)
+        ^"\n//]]>\n" in
+      XML.EncodedPCDATA s'
+    
+    let cdata_style s = (* GK *)
+      (* For security reasons, we do not allow "]]>" inside CDATA
+         (as this string is to be considered as the end of the cdata)
+       *)
+      let s' = "\n/* <![CDATA[ */\n"^
+        (Netstring_pcre.global_replace 
+           (Netstring_pcre.regexp_string "]]>") "" s)
+        ^"\n/* ]]> */\n" in
+      XML.EncodedPCDATA s'
+    
+    let unsafe_data s = XML.EncodedPCDATA s
+    
+
     module TEXT =
       struct
         type heading = [ `H1 | `H2 | `H3 | `H4 | `H5 | `H6 ]
@@ -1546,6 +1595,7 @@ module Version =
     let a_nohref `Nohref = string_attrib "nohref" "nohref"
     let a_usemap = string_attrib "coords"
     let a_defer `Defer = string_attrib "defer" "defer"
+    let a_label = string_attrib "label"
 
     let area ~alt ?(a = []) () = 
       XML.leaf ~a:(a_alt alt :: a) "area"
@@ -1595,7 +1645,8 @@ module Version =
     let button = star "button"
     let legend = star "legend"
     let fieldset = star "fieldset"
-    let optgroup = plus "optgroup"
+    let optgroup ~label ?(a = []) elt elts =
+      XML.node ~a:(a_label label :: a) "optgroup" (elt :: elts)
 
     module TABLES =
       struct
@@ -1828,20 +1879,10 @@ module Version =
     (******************************************************************)
     (* In the following, my own stuffs for Ocsigen -- Vincent: *)
 
-    let blocktags = [ "fieldset"; "form"; "address"; "body"; "head"; "blockquote"; "div"; "html"; "h1"; "h2"; "h3"; "h4"; "h5"; "h6"; "p"; "dd"; "dl"; "li"; "ol"; "ul"; "colgroup"; "table"; "tbody"; "tfoot"; "thead"; "td"; "th"; "tr" ]
-
-    let semiblocktags = [ "pre"; "style"; "title" ]
-
     let tot x = x
     let totl x = x
     let toelt x = x
     let toeltl x = x
-
-    let ocsigen_print version ?width ?encode arbre =
-      XML.xh_print ?width ?encode blocktags semiblocktags (doctype version) arbre
-
-    let ocsigen_xprint version ?width ?encode foret =
-      XML.x_print ?width ?encode blocktags semiblocktags (doctype version) foret
         
   end    
 
@@ -1854,8 +1895,6 @@ module M_01_00 : T_01_00 =
     let standard = M.standard xhtml_version
     let output = M.output xhtml_version
     let pretty_print = M.pretty_print xhtml_version
-    let ocsigen_print = M.ocsigen_print xhtml_version
-    let ocsigen_xprint = M.ocsigen_xprint xhtml_version
     let validator_icon () = M.validator_icon xhtml_version
     let all_anchors elt =
       M.all_anchors elt @ XML.all_string_attribs ~is_elt:((=) "a") "name" elt
@@ -1870,8 +1909,6 @@ module M_01_01 : T_01_01 =
     let standard = M.standard xhtml_version
     let output = M.output xhtml_version
     let pretty_print = M.pretty_print xhtml_version
-    let ocsigen_print = M.ocsigen_print xhtml_version
-    let ocsigen_xprint = M.ocsigen_xprint xhtml_version
     let validator_icon () = M.validator_icon xhtml_version
   end
 
