@@ -1,6 +1,7 @@
 (* $Id: xHTML.ml,v 1.31 2005/06/20 17:57:58 ohl Exp $
 
    Copyright (C) 2004 by Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
+   Copyright (C) 2007 by Vincent Balat, Gabriel Kerneis, CNRS, Universit Paris Diderot
 
    XHTML is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by 
@@ -22,7 +23,7 @@
    BUT: This hack is not very clean because I don't think I follow
    Thorsten Ohl's conventions.
    I verified lots of types but not all and I am not sure that it
-   corresponds exactly to the dtd. (Actually, I am almost sur that it doesn't).
+   corresponds exactly to the dtd. (Actually, I am almost sure that it doesn't).
    This module would deserve a complete verification and to be cleaned 
    following Ohl's conventions.
    Vincent Balat
@@ -148,7 +149,7 @@ module type T =
     type linktypes =
         [ `Alternate | `Appendix | `Bookmark | `Chapter | `Contents
         | `Copyright | `Glossary | `Help | `Index | `Next | `Prev
-        | `Section | `Start | `Stylesheet | `Subsection] list
+        | `Section | `Start | `Stylesheet | `Subsection | `Other of string] list
 (** Authors may use the following recognized link types, listed here with
     their conventional interpretations. A LinkTypes value refers to a
     space-separated list of link types. White space characters are not
@@ -205,7 +206,10 @@ module type T =
          Refers to a bookmark. A bookmark is a link to a key entry point within
          an extended document. The title attribute may be used, for example, to
          label the bookmark. Note that several bookmarks may be defined in each
-         document.}} *)
+         document.}
+      {- [`Other]:
+         refers to any other type (for example [icon] or [shortcut]).
+         }} *)
 
     type mediadesc =
         [ `All | `Aural | `Braille | `Handheld | `Print
@@ -303,7 +307,7 @@ module type T =
 (** Arbitrary textual data, likely meant to be human-readable. *)
 
     type uri (* I abstract this for Ocsigen -- VB *)
-    val make_uri_from_string : string -> uri
+    val uri_of_string : string -> uri
 (** A Uniform Resource Identifier, as per RFC2396.
     @see <http://www.ietf.org/rfc/rfc2396.txt> RFC2396 *)
 
@@ -558,6 +562,8 @@ module type T =
     val a_button_type : [< `Button | `Submit | `Reset ] ->
       [>`Button_Type] attrib
 
+    val a_label : text -> [> `Label ] attrib
+
 
 (** {2 5.6. Table Modules} *)
     
@@ -780,6 +786,14 @@ module type T =
     val pcdata : string -> [>`PCDATA] elt
     val entity : string -> [>`PCDATA] elt
     val space : unit -> [>`PCDATA] elt
+    val cdata : string -> [`PCDATA] elt (* GK *)
+    val cdata_script : string -> [`PCDATA] elt (* GK *)
+    val cdata_style : string -> [`PCDATA] elt (* GK *)
+(**/**)
+    val unsafe_data : string -> [`PCDATA] elt
+(**/**)
+
+
     
 (** {2 Text} *)
     
@@ -878,7 +892,7 @@ module type T =
 
 (** {3 Forms} *)
     
-(** Generic forms. WARNING: I have done this very quickly! This is not valid! -- VB *) 
+(** Generic forms. WARNING: If you find a bug or if something is missing please send a bug report to the Ocsigen project! -- VB *) 
     val form : action:uri ->
       ([< common | `Enctype | `Method | `Name_01_00 | `Target | `Accept_charset | `Accept ],
        [< block_sans_form | `Fieldset ], [>`Form]) plus
@@ -886,8 +900,9 @@ module type T =
   | `Src | `Tabindex | `Input_Type | `Value | `Disabled | `Readonly | `Alt | `Accept | `Usemap ], [>`Input]) nullary
     val label : ([< common | `Accesskey | `For ],
                  [< `PCDATA | inline_sans_label ], [>`Label]) star
-    val optgroup : ([< common | `Disabled | `Label ],
-                  [< `Option ], [>`Optgroup]) plus
+    val optgroup : label:text ->
+      ([< common | `Disabled ],
+       [< `Option ], [>`Optgroup]) plus
     val option : ([< common | `Selected | `Value | `Disabled | `Label ],
                   [< `PCDATA ], [>`Option]) unary
     val select : ([< common | `Multiple | `Name | `Size | `Tabindex | `Disabled ],
@@ -1016,6 +1031,10 @@ module type T =
 (** [~encoding] is the official name of the external character set encoding that
     is used by [outs : string -> unit]. *)
 
+    val doctype :
+        [< `HTML_v03_02 | `HTML_v04_01 | `XHTML_01_00 | `XHTML_01_01 ]
+      -> string
+ 
     val output : ?encode:(string -> string) -> ?encoding:string -> 
       (string -> unit) -> html -> unit
 
@@ -1069,12 +1088,6 @@ module type T =
     val toelt : 'a elt -> XML.elt
     val toeltl : 'a elt list -> XML.elt list
 
-    val ocsigen_print : 
-        ?width:int -> ?encode:(string -> string) -> [ `Html ] elt -> string
-
-    val ocsigen_xprint : 
-        ?width:int -> ?encode:(string -> string) -> 'a elt list -> string
-
   end
 
 (** An alias for XHTML 1.1 (for symmetry):
